from db_dumper import load, dump


def dump_object(obj):
    return {x: getattr(obj, x) for x in dir(obj) if not x.startswith('_')}


class DummyObject:

    def __init__(self, name, age):
        self.name = name
        self.age = age

    def __eq__(self, instance):
        return instance.name == self.name and instance.age == self.age


def test_dump():
    o = DummyObject('Chris Norman', 28)
    data = dump([o], dump_object)
    assert len(data) is 1
    assert list(data.keys()) == [DummyObject.__name__]
    assert list(
        data.values()
    )[0][0] == {
        'name': o.name,
        'age': o.age
    }


def test_load():
    me = DummyObject('Chris Norman', 28)
    dog = DummyObject('Fliss', 10)
    d = dump([me, dog], dump_object)
    objects = load(d, [DummyObject])
    assert len(objects) is 2
    loaded_me, loaded_dog = objects
    assert loaded_me == me
    assert loaded_dog == dog



"""Load and dump databases to disk."""


def load(source, classes):
    """Load 0 or more objects from source and instantiate them with the classes
    found in the classes iterable. They will be returned as a list which can be
    passed to session.add_all for example. No saving will be performed. Pass a
    dictionary as source with name: list-of-dictionaries as pairs."""
    objects = []  # All the created objects.
    classes = {x.__name__: x for x in classes}
    for name, data in source.items():
        cls = classes[name]
        for datum in data:
            # Let's load.
            objects.append(cls(**datum))
    return objects


def dump(objects, f):
    """Return a dictionary suitable for loading with the load function. Objects
    will be dumped with the f function."""
    d = {}
    for obj in objects:
        name = obj.__class__.__name__
        d[name] = d.get(name, [])
        d[name].append(f(obj))
    return d


__all__ = ['load', 'dump']
